defmodule DecentralisedBookIndexWeb.AuthorLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <.header class="pb-2">
      Listing Authors
    </.header>

    <.author_search
      search_query={@search_query}
      select_options={@select_options}
      selected_option={@sort_by}
    />

    <.table
      id="authors"
      rows={@streams.authors}
      row_click={fn {_id, author} -> JS.navigate(~p"/authors/#{author}") end}
    >
      <:col :let={{_id, author}} label="Id">{author.id}</:col>

      <:col :let={{_id, author}} label="Name">{author.name}</:col>

      <:col :let={{_id, author}} label="Description">{author.description}</:col>

      <:col :let={{_id, author}} label="Avatar url">{author.avatar_url}</:col>

      <:action :let={{_id, author}}>
        <div class="sr-only">
          <.link navigate={~p"/authors/#{author}"}>Show</.link>
        </div>

        <.link patch={~p"/authors/#{author}/edit"}>Edit</.link>
      </:action>

      <:action :let={{id, author}}>
        <.link
          phx-click={JS.push("delete", value: %{id: author.id}) |> hide("##{id}")}
          data-confirm="Are you sure?"
        >
          Delete
        </.link>
      </:action>
    </.table>

    <.modal
      :if={@live_action in [:new, :edit]}
      id="author-modal"
      show
      on_cancel={JS.patch(~p"/authors")}
    >
      <.live_component
        module={DecentralisedBookIndexWeb.AuthorLive.FormComponent}
        id={(@author && @author.id) || :new}
        title={@page_title}
        current_user={@current_user}
        action={@live_action}
        author={@author}
        patch={~p"/authors"}
      />
    </.modal>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok,
     socket
     |> stream(
       :authors,
       Ash.read!(DecentralisedBookIndex.Metadata.Author, actor: socket.assigns[:current_user])
     )
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "name") |> validate_sort_by()
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page = Metadata.search_author!(
      search_query,
      page: page_params,
    )

    socket =
      socket
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:select_options, sort_options())
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:params, params)
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Author")
    |> assign(
      :author,
      Ash.get!(DecentralisedBookIndex.Metadata.Author, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Author")
    |> assign(:author, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Authors")
    |> assign(:author, nil)
  end

  @impl true
  def handle_info({DecentralisedBookIndexWeb.AuthorLive.FormComponent, {:saved, author}}, socket) do
    {:noreply, stream_insert(socket, :authors, author)}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    author =
      Ash.get!(DecentralisedBookIndex.Metadata.Author, id, actor: socket.assigns.current_user)

    Ash.destroy!(author, actor: socket.assigns.current_user)

    {:noreply, stream_delete(socket, :authors, author)}
  end

  defp sort_options do
    [
      {"Sort by name", "name"},
      {"Sort by recently updated", "-updated_at"},
      {"Sort by recently added", "-inserted_at"},
    ]
  end

  def validate_sort_by(key) do
    valid_keys = Enum.map(sort_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end

  @impl true
  def handle_event("change-sort", %{"sort_by" => sort_by}, socket) do
    params =
      socket.assigns.params
      |> Map.put("sort_by", sort_by)

    {:noreply, push_patch(socket, to: ~p"/authors?#{params}")}
  end
end
